@@ -30,10 +30,9 @@
 
 /**
  * HdyDeckTransitionType:
- * @HDY_DECK_TRANSITION_TYPE_NONE: No transition
- * @HDY_DECK_TRANSITION_TYPE_SLIDE: Slide from left, right, up or down according to the orientation, text direction and the children order
  * @HDY_DECK_TRANSITION_TYPE_OVER: Cover the old page or uncover the new page, sliding from or towards the end according to orientation, text direction and children order
  * @HDY_DECK_TRANSITION_TYPE_UNDER: Uncover the new page or cover the old page, sliding from or towards the start according to orientation, text direction and children order
+ * @HDY_DECK_TRANSITION_TYPE_SLIDE: Slide from left, right, up or down according to the orientation, text direction and the children order
  *
  * This enumeration value describes the possible transitions between children
  * in a #HdyDeck widget.
@@ -145,23 +144,20 @@ hdy_deck_get_transition_type (HdyDeck *self)
 {
   HdyStackableBoxTransitionType type;
 
-  g_return_val_if_fail (HDY_IS_DECK (self), HDY_DECK_TRANSITION_TYPE_NONE);
+  g_return_val_if_fail (HDY_IS_DECK (self), HDY_DECK_TRANSITION_TYPE_OVER);
 
   type = hdy_stackable_box_get_transition_type (HDY_GET_HELPER (self));
 
   switch (type) {
-  case HDY_STACKABLE_BOX_TRANSITION_TYPE_NONE:
-    return HDY_DECK_TRANSITION_TYPE_NONE;
-
-  case HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE:
-    return HDY_DECK_TRANSITION_TYPE_SLIDE;
-
   case HDY_STACKABLE_BOX_TRANSITION_TYPE_OVER:
     return HDY_DECK_TRANSITION_TYPE_OVER;
 
   case HDY_STACKABLE_BOX_TRANSITION_TYPE_UNDER:
     return HDY_DECK_TRANSITION_TYPE_UNDER;
 
+  case HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE:
+    return HDY_DECK_TRANSITION_TYPE_SLIDE;
+
   default:
     g_assert_not_reached ();
   }
@@ -188,17 +184,9 @@ hdy_deck_set_transition_type (HdyDeck               *self,
   HdyStackableBoxTransitionType type;
 
   g_return_if_fail (HDY_IS_DECK (self));
-  g_return_if_fail (transition <= HDY_DECK_TRANSITION_TYPE_UNDER);
+  g_return_if_fail (transition <= HDY_DECK_TRANSITION_TYPE_SLIDE);
 
   switch (transition) {
-  case HDY_DECK_TRANSITION_TYPE_NONE:
-    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_NONE;
-    break;
-
-  case HDY_DECK_TRANSITION_TYPE_SLIDE:
-    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE;
-    break;
-
   case HDY_DECK_TRANSITION_TYPE_OVER:
     type = HDY_STACKABLE_BOX_TRANSITION_TYPE_OVER;
     break;
@@ -207,6 +195,10 @@ hdy_deck_set_transition_type (HdyDeck               *self,
     type = HDY_STACKABLE_BOX_TRANSITION_TYPE_UNDER;
     break;
 
+  case HDY_DECK_TRANSITION_TYPE_SLIDE:
+    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE;
+    break;
+
   default:
     g_assert_not_reached ();
   }
@@ -705,8 +697,6 @@ hdy_deck_finalize (GObject *object)
 
   g_clear_object (&priv->box);
 
-  g_object_set_data (object, "captured-event-handler", NULL);
-
   G_OBJECT_CLASS (hdy_deck_parent_class)->finalize (object);
 }
 
@@ -828,13 +818,6 @@ hdy_deck_get_cancel_progress (HdySwipeable *swipeable)
   return hdy_stackable_box_get_cancel_progress (HDY_GET_HELPER (swipeable));
 }
 
-static gboolean
-captured_event_cb (HdyDeck  *self,
-                   GdkEvent *event)
-{
-  return hdy_stackable_box_captured_event (HDY_GET_HELPER (self), event);
-}
-
 static void
 hdy_deck_class_init (HdyDeckClass *klass)
 {
@@ -941,7 +924,7 @@ hdy_deck_class_init (HdyDeckClass *klass)
     g_param_spec_enum ("transition-type",
                        _("Transition type"),
                        _("The type of animation used to transition between children"),
-                       HDY_TYPE_DECK_TRANSITION_TYPE, HDY_DECK_TRANSITION_TYPE_NONE,
+                       HDY_TYPE_DECK_TRANSITION_TYPE, HDY_DECK_TRANSITION_TYPE_OVER,
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
@@ -1080,14 +1063,6 @@ hdy_deck_init (HdyDeck *self)
   g_signal_connect_object (priv->box, "notify::can-swipe-back", G_CALLBACK (notify_can_swipe_back_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->box, "notify::can-swipe-forward", G_CALLBACK (notify_can_swipe_forward_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->box, "notify::orientation", G_CALLBACK (notify_orientation_cb), self, G_CONNECT_SWAPPED);
-
-  /*
-   * HACK: GTK3 has no other way to get events on capture phase.
-   * This is a reimplementation of _gtk_widget_set_captured_event_handler(),
-   * which is private. In GTK4 it can be replaced with GtkEventControllerLegacy
-   * with capture propagation phase
-   */
-  g_object_set_data (G_OBJECT (self), "captured-event-handler", captured_event_cb);
 }
 
 static void
