@@ -34,10 +34,9 @@
 
 /**
  * HdyLeafletTransitionType:
- * @HDY_LEAFLET_TRANSITION_TYPE_NONE: No transition
- * @HDY_LEAFLET_TRANSITION_TYPE_SLIDE: Slide from left, right, up or down according to the orientation, text direction and the children order
  * @HDY_LEAFLET_TRANSITION_TYPE_OVER: Cover the old page or uncover the new page, sliding from or towards the end according to orientation, text direction and children order
  * @HDY_LEAFLET_TRANSITION_TYPE_UNDER: Uncover the new page or cover the old page, sliding from or towards the start according to orientation, text direction and children order
+ * @HDY_LEAFLET_TRANSITION_TYPE_SLIDE: Slide from left, right, up or down according to the orientation, text direction and the children order
  *
  * This enumeration value describes the possible transitions between modes and
  * children in a #HdyLeaflet widget.
@@ -170,23 +169,20 @@ hdy_leaflet_get_transition_type (HdyLeaflet *self)
 {
   HdyStackableBoxTransitionType type;
 
-  g_return_val_if_fail (HDY_IS_LEAFLET (self), HDY_LEAFLET_TRANSITION_TYPE_NONE);
+  g_return_val_if_fail (HDY_IS_LEAFLET (self), HDY_LEAFLET_TRANSITION_TYPE_OVER);
 
   type = hdy_stackable_box_get_transition_type (HDY_GET_HELPER (self));
 
   switch (type) {
-  case HDY_STACKABLE_BOX_TRANSITION_TYPE_NONE:
-    return HDY_LEAFLET_TRANSITION_TYPE_NONE;
-
-  case HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE:
-    return HDY_LEAFLET_TRANSITION_TYPE_SLIDE;
-
   case HDY_STACKABLE_BOX_TRANSITION_TYPE_OVER:
     return HDY_LEAFLET_TRANSITION_TYPE_OVER;
 
   case HDY_STACKABLE_BOX_TRANSITION_TYPE_UNDER:
     return HDY_LEAFLET_TRANSITION_TYPE_UNDER;
 
+  case HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE:
+    return HDY_LEAFLET_TRANSITION_TYPE_SLIDE;
+
   default:
     g_assert_not_reached ();
   }
@@ -213,17 +209,9 @@ hdy_leaflet_set_transition_type (HdyLeaflet               *self,
   HdyStackableBoxTransitionType type;
 
   g_return_if_fail (HDY_IS_LEAFLET (self));
-  g_return_if_fail (transition <= HDY_LEAFLET_TRANSITION_TYPE_UNDER);
+  g_return_if_fail (transition <= HDY_LEAFLET_TRANSITION_TYPE_SLIDE);
 
   switch (transition) {
-  case HDY_LEAFLET_TRANSITION_TYPE_NONE:
-    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_NONE;
-    break;
-
-  case HDY_LEAFLET_TRANSITION_TYPE_SLIDE:
-    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE;
-    break;
-
   case HDY_LEAFLET_TRANSITION_TYPE_OVER:
     type = HDY_STACKABLE_BOX_TRANSITION_TYPE_OVER;
     break;
@@ -232,6 +220,10 @@ hdy_leaflet_set_transition_type (HdyLeaflet               *self,
     type = HDY_STACKABLE_BOX_TRANSITION_TYPE_UNDER;
     break;
 
+  case HDY_LEAFLET_TRANSITION_TYPE_SLIDE:
+    type = HDY_STACKABLE_BOX_TRANSITION_TYPE_SLIDE;
+    break;
+
   default:
     g_assert_not_reached ();
   }
@@ -769,8 +761,6 @@ hdy_leaflet_finalize (GObject *object)
 
   g_clear_object (&priv->box);
 
-  g_object_set_data (object, "captured-event-handler", NULL);
-
   G_OBJECT_CLASS (hdy_leaflet_parent_class)->finalize (object);
 }
 
@@ -901,13 +891,6 @@ hdy_leaflet_get_cancel_progress (HdySwipeable *swipeable)
   return hdy_stackable_box_get_cancel_progress (HDY_GET_HELPER (swipeable));
 }
 
-static gboolean
-captured_event_cb (HdyLeaflet *self,
-                   GdkEvent   *event)
-{
-  return hdy_stackable_box_captured_event (HDY_GET_HELPER (self), event);
-}
-
 static void
 hdy_leaflet_class_init (HdyLeafletClass *klass)
 {
@@ -1185,14 +1168,6 @@ hdy_leaflet_init (HdyLeaflet *self)
   g_signal_connect_object (priv->box, "notify::can-swipe-back", G_CALLBACK (notify_can_swipe_back_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->box, "notify::can-swipe-forward", G_CALLBACK (notify_can_swipe_forward_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->box, "notify::orientation", G_CALLBACK (notify_orientation_cb), self, G_CONNECT_SWAPPED);
-
-  /*
-   * HACK: GTK3 has no other way to get events on capture phase.
-   * This is a reimplementation of _gtk_widget_set_captured_event_handler(),
-   * which is private. In GTK4 it can be replaced with GtkEventControllerLegacy
-   * with capture propagation phase
-   */
-  g_object_set_data (G_OBJECT (self), "captured-event-handler", captured_event_cb);
 }
 
 static void
