@@ -33,9 +33,6 @@ gboolean         hdy_swipe_tracker_get_allow_mouse_drag (HdySwipeTracker *self);
 void             hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
                                                          gboolean         allow_mouse_drag);
 
-gboolean         hdy_swipe_tracker_captured_event (HdySwipeTracker *self,
-                                                   GdkEvent        *event);
-
 void             hdy_swipe_tracker_shift_position (HdySwipeTracker *self,
                                                    gdouble          delta);
 
