@@ -7,6 +7,7 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
+#include "hdy-leaflet.h"
 #include "hdy-swipe-tracker-private.h"
 #include "hdy-navigation-direction.h"
 
@@ -33,27 +34,6 @@
  * gestures. It supports touch-based swipes, pointer dragging, and touchpad
  * scrolling.
  *
- * The events must be received as early as possible to defer the events to
- * child widgets when needed. Usually this happens naturally, but
- * GtkScrolledWindow receives events on capture phase via a private function.
- * Because of that, implementing widgets must do the same thing, i.e. receive
- * events on capture phase and call hdy_swipe_tracker_captured_event() for
- * each event. This can be done as follows:
- * |[<!-- language="C" -->
- * g_object_set_data (G_OBJECT (self), "captured-event-handler", captured_event_cb);
- * ]|
- * Where captured_event_cb() is:
- * |[<!-- language="C" -->
- * static gboolean
- * captured_event_cb (MyWidget *self, GdkEvent *event)
- * {
- *   return hdy_swipe_tracker_captured_event (self->tracker, event);
- * }
- * ]|
- *
- * NOTE: In GTK4 this can be replaced by a GtkEventControllerLegacy with capture
- * propagation phase.
- *
  * The widgets will probably want to expose #HdySwipeTracker:enabled property.
  * If they expect to use horizontal orientation, #HdySwipeTracker:reversed
  * property can be used for supporting RTL text direction.
@@ -381,8 +361,9 @@ drag_cancel_cb (HdySwipeTracker  *self,
 }
 
 static gboolean
-captured_scroll_event (HdySwipeTracker *self,
-                       GdkEvent        *event)
+handle_scroll_event (HdySwipeTracker *self,
+                     GdkEvent        *event,
+                     gboolean         capture)
 {
   GdkDevice *source_device;
   GdkInputSource input_source;
@@ -421,15 +402,16 @@ captured_scroll_event (HdySwipeTracker *self,
     if (gdk_event_is_scroll_stop_event (event))
       return GDK_EVENT_PROPAGATE;
 
-    if (is_vertical == is_delta_vertical)
-      gesture_prepare (self, delta > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK);
-    else {
+    if (is_vertical == is_delta_vertical) {
+      if (!capture)
+        gesture_prepare (self, delta > 0 ? HDY_NAVIGATION_DIRECTION_FORWARD : HDY_NAVIGATION_DIRECTION_BACK);
+    } else {
       self->is_scrolling = TRUE;
       return GDK_EVENT_PROPAGATE;
     }
   }
 
-  if (self->state == HDY_SWIPE_TRACKER_STATE_PENDING) {
+  if (!capture && self->state == HDY_SWIPE_TRACKER_STATE_PENDING) {
     gboolean is_overshooting;
     gdouble first_point, last_point;
 
@@ -453,12 +435,102 @@ captured_scroll_event (HdySwipeTracker *self,
     }
   }
 
-  if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING)
+  if (!capture && self->state == HDY_SWIPE_TRACKER_STATE_FINISHING)
     reset (self);
 
   return GDK_EVENT_PROPAGATE;
 }
 
+static gboolean
+is_window_handle (GtkWidget *widget)
+{
+  gboolean window_dragging;
+  GtkWidget *parent, *window, *titlebar;
+
+  gtk_widget_style_get (widget, "window-dragging", &window_dragging, NULL);
+
+  if (window_dragging)
+    return TRUE;
+
+  /* Window titlebar area is always draggable, so check if we're inside. */
+  window = gtk_widget_get_toplevel (widget);
+  if (!GTK_IS_WINDOW (window))
+    return FALSE;
+
+  titlebar = gtk_window_get_titlebar (GTK_WINDOW (window));
+  if (!titlebar)
+    return FALSE;
+
+  parent = widget;
+  while (parent && parent != titlebar)
+    parent = gtk_widget_get_parent (parent);
+
+  return parent == titlebar;
+}
+
+static gboolean
+handle_event_cb (HdySwipeTracker *self,
+                 GdkEvent        *event)
+{
+  GdkEventSequence *sequence;
+  gboolean retval;
+  GtkEventSequenceState state;
+  GtkWidget *widget;
+
+  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
+    return GDK_EVENT_PROPAGATE;
+
+  if (event->type == GDK_SCROLL)
+    return handle_scroll_event (self, event, FALSE);
+
+  if (event->type != GDK_BUTTON_PRESS &&
+      event->type != GDK_BUTTON_RELEASE &&
+      event->type != GDK_MOTION_NOTIFY &&
+      event->type != GDK_TOUCH_BEGIN &&
+      event->type != GDK_TOUCH_END &&
+      event->type != GDK_TOUCH_UPDATE &&
+      event->type != GDK_TOUCH_CANCEL)
+    return GDK_EVENT_PROPAGATE;
+
+  widget = gtk_get_event_widget (event);
+  if (is_window_handle (widget))
+    return GDK_EVENT_PROPAGATE;
+
+  sequence = gdk_event_get_event_sequence (event);
+  retval = gtk_event_controller_handle_event (GTK_EVENT_CONTROLLER (self->touch_gesture), event);
+  state = gtk_gesture_get_sequence_state (self->touch_gesture, sequence);
+
+  if (state == GTK_EVENT_SEQUENCE_DENIED) {
+    gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
+    return GDK_EVENT_PROPAGATE;
+  }
+
+  if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
+    return GDK_EVENT_STOP;
+  } else if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING) {
+    reset (self);
+    return GDK_EVENT_STOP;
+  }
+  return retval;
+}
+
+static gboolean
+captured_event_cb (HdySwipeable *swipeable,
+                   GdkEvent     *event)
+{
+  HdySwipeTracker *self = g_object_get_data (G_OBJECT (swipeable), "swipe-tracker");
+
+  g_assert (HDY_IS_SWIPE_TRACKER (self));
+
+  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
+    return GDK_EVENT_PROPAGATE;
+
+  if (event->type != GDK_SCROLL)
+    return GDK_EVENT_PROPAGATE;
+
+  return handle_scroll_event (self, event, TRUE);
+}
+
 static void
 hdy_swipe_tracker_constructed (GObject *object)
 {
@@ -484,6 +556,18 @@ hdy_swipe_tracker_constructed (GObject *object)
   g_signal_connect_swapped (self->touch_gesture, "drag-end", G_CALLBACK (drag_end_cb), self);
   g_signal_connect_swapped (self->touch_gesture, "cancel", G_CALLBACK (drag_cancel_cb), self);
 
+  g_signal_connect_object (self->swipeable, "event", G_CALLBACK (handle_event_cb), self, G_CONNECT_SWAPPED);
+
+  g_object_set_data (G_OBJECT (self->swipeable), "swipe-tracker", self);
+
+  /*
+   * HACK: GTK3 has no other way to get events on capture phase.
+   * This is a reimplementation of _gtk_widget_set_captured_event_handler(),
+   * which is private. In GTK4 it can be replaced with GtkEventControllerLegacy
+   * with capture propagation phase
+   */
+  g_object_set_data (G_OBJECT (self->swipeable), "captured-event-handler", captured_event_cb);
+
   G_OBJECT_CLASS (hdy_swipe_tracker_parent_class)->constructed (object);
 }
 
@@ -498,6 +582,9 @@ hdy_swipe_tracker_dispose (GObject *object)
   if (self->touch_gesture)
     g_signal_handlers_disconnect_by_data (self->touch_gesture, self);
 
+  g_object_set_data (G_OBJECT (self->swipeable), "swipe-tracker", NULL);
+  g_object_set_data (G_OBJECT (self->swipeable), "captured-event-handler", NULL);
+
   g_clear_object (&self->touch_gesture);
   g_clear_object (&self->swipeable);
 
@@ -675,8 +762,14 @@ hdy_swipe_tracker_init (HdySwipeTracker *self)
 HdySwipeTracker *
 hdy_swipe_tracker_new (HdySwipeable *swipeable)
 {
+  gpointer swipe_tracker;
+
   g_return_val_if_fail (swipeable != NULL, NULL);
 
+  swipe_tracker = g_object_get_data (G_OBJECT (swipeable), "swipe-tracker");
+
+  g_return_val_if_fail (swipe_tracker == NULL, NULL);
+
   return g_object_new (HDY_TYPE_SWIPE_TRACKER,
                        "swipeable", swipeable,
                        NULL);
@@ -820,93 +913,6 @@ hdy_swipe_tracker_set_allow_mouse_drag (HdySwipeTracker *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ALLOW_MOUSE_DRAG]);
 }
 
-static gboolean
-is_window_handle (GtkWidget *widget)
-{
-  gboolean window_dragging;
-  GtkWidget *parent, *window, *titlebar;
-
-  gtk_widget_style_get (widget, "window-dragging", &window_dragging, NULL);
-
-  if (window_dragging)
-    return TRUE;
-
-  /* Window titlebar area is always draggable, so check if we're inside. */
-  window = gtk_widget_get_toplevel (widget);
-  if (!GTK_IS_WINDOW (window))
-    return FALSE;
-
-  titlebar = gtk_window_get_titlebar (GTK_WINDOW (window));
-  if (!titlebar)
-    return FALSE;
-
-  parent = widget;
-  while (parent && parent != titlebar)
-    parent = gtk_widget_get_parent (parent);
-
-  return parent == titlebar;
-}
-
-/**
- * hdy_swipe_tracker_captured_event:
- * @self: a #HdySwipeTracker
- * @event: a captured #GdkEvent
- *
- * Handles an event. This must be called for events received at capture phase
- * only.
- *
- * Returns: %TRUE is the event was handled and must not be propagated
- *
- * Since: 0.0.11
- */
-gboolean
-hdy_swipe_tracker_captured_event (HdySwipeTracker *self,
-                                  GdkEvent        *event)
-{
-  GdkEventSequence *sequence;
-  gboolean retval;
-  GtkEventSequenceState state;
-  GtkWidget *widget;
-
-  g_return_val_if_fail (HDY_IS_SWIPE_TRACKER (self), GDK_EVENT_PROPAGATE);
-
-  if (!self->enabled && self->state != HDY_SWIPE_TRACKER_STATE_SCROLLING)
-    return GDK_EVENT_PROPAGATE;
-
-  if (event->type == GDK_SCROLL)
-    return captured_scroll_event (self, event);
-
-  if (event->type != GDK_BUTTON_PRESS &&
-      event->type != GDK_BUTTON_RELEASE &&
-      event->type != GDK_MOTION_NOTIFY &&
-      event->type != GDK_TOUCH_BEGIN &&
-      event->type != GDK_TOUCH_END &&
-      event->type != GDK_TOUCH_UPDATE &&
-      event->type != GDK_TOUCH_CANCEL)
-    return GDK_EVENT_PROPAGATE;
-
-  widget = gtk_get_event_widget (event);
-  if (is_window_handle (widget))
-    return GDK_EVENT_PROPAGATE;
-
-  sequence = gdk_event_get_event_sequence (event);
-  retval = gtk_event_controller_handle_event (GTK_EVENT_CONTROLLER (self->touch_gesture), event);
-  state = gtk_gesture_get_sequence_state (self->touch_gesture, sequence);
-
-  if (state == GTK_EVENT_SEQUENCE_DENIED) {
-    gtk_event_controller_reset (GTK_EVENT_CONTROLLER (self->touch_gesture));
-    return GDK_EVENT_PROPAGATE;
-  }
-
-  if (self->state == HDY_SWIPE_TRACKER_STATE_SCROLLING) {
-    return GDK_EVENT_STOP;
-  } else if (self->state == HDY_SWIPE_TRACKER_STATE_FINISHING) {
-    reset (self);
-    return GDK_EVENT_STOP;
-  }
-  return retval;
-}
-
 void
 hdy_swipe_tracker_shift_position (HdySwipeTracker *self,
                                   gdouble          delta)
